import pygame
import data

class surf(object):
    def __init__(self,surface,dp=[0,0]):
        self.surface = surface
        self.dp = dp
    def __getattr__(self,key):
        if key in dir(self.surface):
            return getattr(self.surface,key)
        raise AttributeError(key)
    def paint(self,dest,pos):
        pos = [pos[i]+self.dp[i] for i in range(2)]
        dest.blit(self.surface,pos)

class Sprite:
    def __init__(self):
        self.images = []  #The currently displayed surfaces
        self.pos = [0,0]  #Position to display
        self.size = [0,0] #Size according to image
        self.last_pos_drawn = None  #Only redraw if position changes
        self.dirty = True
        self.behind = None  #Surface of what was behind us before we were
                            #pasted
        self.priority = 0   #higher number = higher layer
    def has_changed(self):
        return self.dirty or \
                self.pos != self.last_pos_drawn
    def loadimage(self,filename):
        """By default, replaces the image list with [loaded_image]"""
        oi = self.images[:]
        self.images = [surf(data.loadimage(filename))]
        self.size = self.images[0].get_size()
        #Set the image as dirty so renderer redraws it
        if oi!=self.images:
            self.dirty = True

class Man(Sprite):
    def __init__(self,job):
        Sprite.__init__(self)
        self.job = job
        self.loc = 0
        self.carry = None
        self.destination = None
        self.task = None

        self.dir = 1
        self.speed = 30

        self.clearcol = clearcol = [246,255,167]
        self.main_img = surf(data.loadimage('peoples/%s.png'%job,False))
        self.main_img.set_colorkey(clearcol)
        self.choke_img = surf(data.loadimage('peoples/%schoke.png'%job,False))
        self.choke_img.set_colorkey(clearcol)
        self.dead_img = surf(data.loadimage('peoples/%sdead.png'%job,False))
        self.dead_img.set_colorkey(clearcol)
        self.num_frames = 0

        self.animspeed = 100
        self.next_frame = self.animspeed
        self.loop = True

        self.alive = True
        self.choke_sound = pygame.mixer.Sound(data.load('sounds/wuerg.wav'))

        self.set_frame(0,"main")
    def add_frame(self,dt):
        self.next_frame -= dt
        if self.next_frame <= 0:
            self.next_frame = self.animspeed
            self.frame += 1
            if self.frame>self.num_frames:
                if self.loop:
                    self.frame = 0
                else:
                    return
            self.set_frame(self.frame)
    def set_frame(self,frame,anim=None,loop=None):
        if loop != None:
            self.loop = loop
        if anim:
            anim = getattr(self,anim+"_img")
            if anim!=getattr(self,"anim",None):
                self.dirty = True
                self.anim = anim
                self.num_frames = self.anim.get_width()/30 - 1
        if frame != getattr(self,"frame",None):
            self.frame = frame
            self.dirty = True
        i = self.anim.subsurface([[self.frame*30,0],[30,30]])
        if self.dir<0:
            i = pygame.transform.flip(i,-1,0)
        i.set_colorkey(self.clearcol)
        self.images = [surf(i)]
        self.size = [30,30]
    def mouse_click(self):
        if hasattr(self,"click_event"):
            self.click_event()
    def mouse_over(self):
        pass
    def mouse_exit(self):
        pass


class Text(Sprite):
    def updatetext(self,text='test'):
        self.font = pygame.font.SysFont(pygame.font.get_default_font(),
                48)
        self.text = text
    def loadimage(self,filename):
        if not hasattr(self,"filename"):
            self.filename = filename
            self.filenameh = filename.replace(".png","")+"h.png"
        Sprite.loadimage(self,filename)
        wt = surf(self.font.render(self.text,1,[240,240,255]),[4,6])
        bt = surf(self.font.render(self.text,1,[25,30,40]),[6,8])
        self.images.extend([bt,wt])

class Button(Text):
    def mouse_enter(self):
        self.hi = 1
        self.loadimage(self.filenameh)
    def mouse_exit(self):
        self.hi = 0
        self.loadimage(self.filename)
    def mouse_click(self):
        if hasattr(self,"click_event"):
            self.click_event()
